%% Instructions and exercises for simulating hyperbolic secant adiabatic pulses
%
% Make sure to follow the instructions given at the top of the simulation
% m-file to download and set up the Bloch simulator.
%
% First run the simulation with all default parameters, and study the
% resulting plots. Next, go through the following exercises.
%
% Exercise 1.
% The hyperbolic secant adiabatic inversion pulse has been provided with a
% duration of 10.24 ms. This is the default duration used in our MRI
% scanners. In class, we discussed that adiabatic pulses generally need to
% have long durations, but is this really true? Try to change the duration
% of the pulse (pulseWidth) in Line 26 to convince yourself. Run the 
% simulation with several different durations, between 2 ms and 12 ms, to 
% convice yourself that a long duration is important (hint: look
% at the inversion profile for each different duration). What is the
% minimum duration (approximately) required for 90% inversion?
%
% Exercise 2.
% The hyperbolic secant adiabatic inversion pulse has been provided with a
% peak B1 amplitude of 12 uT. This is close to the default B1 amplitude
% used in our scanners. In class, we discussed that adiabatic pulses 
% generally need to have high amplitudes, but again, is this really true? 
% Try to change the B1 amplitude of the pulse (A0) in Line 27 to convince 
% yourself. Run the simulation with several different different peak B1
% amplitudes, between 2 uT and 15 uT, to convice yourself that a high B1
% amplitude is necessary (hint: look at the inversion profile for each 
% different amplitude). What is the minimum B1 amplitude required 
% (approximately) for 90% inversion?
%
% Exercise 3.
% You should have noticed by now that the Bloch simulation takes inputs
% for T1 and T2 values. For simplicity, T1 and T2 relaxation has been 
% ignored until now by setting the values to 10000. However, let's see how
% T1 and T2 can affect the pulse. Set new values: T1 = 600 ms and T2 = 50
% ms. How do these values affect the inversion profile? The inversion
% profile you see now is a more realistic representation of inversion 
% profile for human tissue such as the liver, which has these T1 and T2
% values. 
%
% Exercise 4.
% Remember to set the T1 and T2 values back to 10000 to ignore relaxation.
% Next, suppose you want to create a new hyperbolic secant inversion pulse.
% This can be done simply by changing the parameters in the RF pulse
% equations. Start by setting mu = 8; in Line 24. How does this affect the 
% pulse waveforms and inversion profile? Next, set beta1 = 990; in Line 25. 
% How does this affect pulse waveforms and inversion profile? Why? Can you 
% think of how you might improve the inversion profile (answers in Exercise
% 5).
%
% Exercise 5.
% In Exercise 4, when you simulate the pulse with mu = 8 and beta = 990,
% the spectral bandwidth (full-width half-max of the inversion profile) 
% changes from 1 kHz to 2.5 kHz. Recall that the spectral bandwidth of the 
% hyperbolic secant inversion pulse is mu * beta / pi. However, the 
% inversion efficiency decreases from 100% to 76% (from inversion profile).
% This is because changing mu and beta also changes the required minimum 
% B1 amplitude for adiabatic passage. Recall that the minimum required 
% B1 amplitude of the hyperbolic secant pulse, A0, is given by:
%     A0 >> beta * sqrt(mu) / (2*pi*gamma)
% For the pulse in Exercise 1, the min. A0 = 5.6 uT. So an A0 of 12 uT was
% sufficient. But in Exercise 4, the min. A0 = 10.5 uT, so an A0 of 12 uT
% is no longer sufficient. For best performance, the actual A0 should be
% double the min. A0.
% Simulate the pulse in Exercise 4, but this time set A0 = 22 uT = 0.22 G 
% (Line 27). How does this affect the pulse waveforms and inversion 
% profile?
% 
% Exercise 6.
% The main purpose of Exercise 5 was to increase the spectral bandwidth of
% the pulse from 1 kHz to 2.5 kHz. As you saw, this can be achieved by
% setting mu = 8, beta = 990 and A0 = 22 uT.
% However, in real life, a peak B1 amplitude of 22 uT may be difficult to
% achieve in the human body. So, is there a way to achieve a 2.5 kHz 
% bandwidth but with a lower B1 amplitude? One way to do this is as 
% follows:
% Remember, the bandwidth increases as beta*mu, but the minimum A0 increases
% as beta*sqrt(mu). So, instead of using mu = 8 and beta = 990, if we use
% the following combination: beta = 672, mu = 11.69, we will still get a
% spectral bandwidth of 2.5 kHz, but now the minimum A0 is 8.6 uT. So, by 
% setting the actual A0 to 16 uT, we should be able to get good performance 
% from the hyperbolic secant pulse.
% Simulate the hyperbolic secant pulse with beta = 672, mu = 11.69 and 
% A0 = 16 uT. 
%
%
%% Further Exercises:
%
% The following exercises are completely optional. Go through them only if
% you want to gain a deeper understanding of adiabatic inversion pulses.
%
% Exercise 7.
% To study the adiabatic property of the pulse with respect to peak B1
% amplitude, it is often useful to look at a plot of the inversion
% profile as it varies with B1 amplitude. Since the inversion profile is
% generally observed as a function of resonance frequency, this requires a
% 3D plot. The B1 amplitude can be scaled directly in the bloch simulation
% code, simply by including a multiplication factor with the rf_pulse
% variable.
%
% Run the following script to generate plots of inversion profile vs. B1
% amplitude. You will need to run the original simulation first to set up
% the RF pulse and the parameters.
%
%% Run this script by copying it to a new m-file, or by pressing CTRL+ENTER

B1Multiplier = linspace(0,2,70);
new_freq_range  = linspace(-2000,2000,120);  % Creating a smaller frequency range to speed up the Bloch simulation
mz2_arr = zeros(length(B1Multiplier),length(new_freq_range));

for tx = 1:length(B1Multiplier)
	[mx2,my2,mz2] = bloch(B1Multiplier(tx).*rf_pulse,  grad_pulse,  dt,  T1_value/1000,  T2_value/1000,  new_freq_range,  0,  0,  0,  0,  0);
	mz2_arr(tx,:) = mz2;
end

% This is one way to view the plot:
figure; imagesc(freq_range,B1Multiplier.*max(abs(rf_pulse)).*100,(mz2_arr),[-1 1]);
title('2D Surface of Inversion Profile vs. B1 Amplitude');
xlabel('Frequency (Hz)'); ylabel('B1 amplitude (\muT)'); colorbar; 

% This is another way. This creates a 3D plot:
[freqmesh, B1mesh] = meshgrid(new_freq_range, B1Multiplier.*max(abs(rf_pulse)).*100);
figure; surf(freqmesh,B1mesh,mz2_arr); 
xlabel('Frequency (Hz)'); ylabel('B1 amplitude (\muT)'); zlabel('M_z'); axis ij; axis vis3d
title('3D Surface of Inversion Profile vs. B1 Amplitude');

% End of script.
%
%
%%
% Exercise 8.
% So far, in all our simulations, we have been looking at plots of Mz after
% the RF pulse ends. But the Bloch simulator that we are using is extremely
% powerful, and it provides Mx and My as well. Furthermore, it can provide
% Mx, My and Mz as a function of time over the application of the RF pulse.
% (You can do this by setting mod = 2 in the Bloch simulation input).
% In other words, we can observe how the magnetization vector changes
% during the application of the RF pulse.
%
% Run the following script to see an animation of how an on-resonance 
% magnetization vector changes during the application of the hyperbolic secant inversion
% pulse. You will see that the magnetization vector starts at [0,0,1] and ends
% at [0,0,-1]. Remember that you are observing the magnetization vector from
% the rotating frame of reference (rotating at Larmor frequency).


%% Run this script by copying it to a new m-file, or by pressing CTRL+ENTER

mod = 2;
[mx1,my1,mz1] = bloch(rf_pulse,  grad_pulse,  dt,  T1_value/1000,  T2_value/1000,  freq_range,  0,  mod,  0,  0,  0);
fig8 = figure('Position',[200,200,710,578]); plot3(0,0,0,'ko'); axis([-1,1,-1,1,-1,1]); grid on; hold on; 
xlabel('M_x'); ylabel('M_y'); zlabel('M_z');
selected_freq = 0;
x1 = 1; x2 = find(freq_range>=selected_freq,1,'first'); 
figure(fig8); h2 = plot3([0,mx1(x1,x2)],[0,my1(x1,x2)],[0,mz1(x1,x2)]);
figure(fig8); h3 = plot3(mx1(x1,x2),my1(x1,x2),mz1(x1,x2),'s');

for t = 1:2:size(mz1,1)
	x1 = t;
 	delete(h2); delete(h3); % Comment out this line and see what happens!
	figure(fig8); h2 = plot3([0,mx1(x1,x2)],[0,my1(x1,x2)],[0,mz1(x1,x2)]);
	figure(fig8); h3 = plot3(mx1(x1,x2),my1(x1,x2),mz1(x1,x2),'rs');
	figure(fig8); title(['Time = ',num2str(t),' ms']); 
	pause(0.03); % adjust this pause value if the animation speed is too fast or too slow
end


%%
% Exercise 9.
% In contrast, run the following script to see an animation of how an
% on-resonance magnetization vector changes during the application of a
% non-adiabatic sinc pulse with flip angle 90 degrees. The sinc pulse used in this example has a duration of 4 ms
% and a TBW of 8. You will need the wsinc and rfscaleg Matlab functions to run
% this script. Again, remember that you are observing the magnetization vector from
% the rotating frame of reference (rotating at Larmor frequency).

%% Run this script by copying it to a new m-file, or by pressing CTRL+ENTER

nSamples = 500;  % number of samples in the RF pulse
pulseWidth = 4;  % [ms]
dt = pulseWidth/nSamples/1000;  % time step, [seconds]

% Set up the sinc pulse and plot it:
rf = pi/2.*wsinc(8,nSamples); rf_sinc = rfscaleg(rf,pulseWidth); t_rf_sinc = linspace(0,4,500);
figure;plot(t_rf_sinc, rf_sinc.*100); grid on; xlabel('Time (ms)'); ylabel('B1 Amplitude (\muT)')

grad_pulse = zeros(1,length(rf_sinc));
mod = 2; T1_value = 1e5; T2_value = 1e5; freq_range  = linspace(-4000,4000,1000); 
[mx3,my3,mz3] = bloch(rf_sinc,  grad_pulse,  dt,  T1_value/1000,  T2_value/1000,  freq_range,  0,  mod,  0,  0,  0);

selected_freq = 0;
x1 = 1; x2 = find(freq_range>=selected_freq,1,'first'); 

fig9 = figure('Position',[200,200,710,578]); plot3(0,0,0,'ko'); axis([-1,1,-1,1,-1,1]); grid on; hold on; 
xlabel('M_x'); ylabel('M_y'); zlabel('M_z');
h2 = plot3([0,mx3(x1,x2)],[0,my3(x1,x2)],[0,mz3(x1,x2)]);
h3 = plot3(mx3(x1,x2),my3(x1,x2),mz3(x1,x2),'s');

for t = 1:1:size(mz3,1)
	x1 = t;
 	delete(h2); delete(h3);
	h2 = plot3([0,mx3(x1,x2)],[0,my3(x1,x2)],[0,mz3(x1,x2)]);
	h3 = plot3(mx3(x1,x2),my3(x1,x2),mz3(x1,x2),'rs');
	title(['Time = ',num2str(t),' ms']); 
	pause(0.03); % adjust this pause value if the animation speed is too fast or too slow
end

%% For more fun, look at some off-resonant magnetization vectors. You can change the resonance frequency by changing selected_freq.

